// SPDX-License-Identifier: GPL-2.0 or Linux-OpenIB
/* Copyright (c) 2024 ZTE Corporation.  All rights reserved. */
#ifndef ZXDH_VERBS_H
#define ZXDH_VERBS_H
#include "zxdh_defs.h"

#define zxdh_handle void *
#define zxdh_adapter_handle zxdh_handle
#define zxdh_qp_handle zxdh_handle
#define zxdh_cq_handle zxdh_handle
#define zxdh_pd_id zxdh_handle
#define zxdh_stag_handle zxdh_handle
#define zxdh_stag_index __u32
#define zxdh_stag __u32
#define zxdh_stag_key __u8
#define zxdh_tagged_offset __u64
#define zxdh_access_privileges __u32
#define zxdh_physical_fragment __u64
#define zxdh_address_list __u64 *
#define zxdh_sgl struct zxdh_sge *

#define ZXDH_MAX_MR_SIZE 0x200000000000ULL

#define ZXDH_ACCESS_FLAGS_LOCALREAD 0x01
#define ZXDH_ACCESS_FLAGS_LOCALWRITE 0x02
#define ZXDH_ACCESS_FLAGS_REMOTEREAD_ONLY 0x04
#define ZXDH_ACCESS_FLAGS_REMOTEREAD 0x05
#define ZXDH_ACCESS_FLAGS_REMOTEWRITE_ONLY 0x08
#define ZXDH_ACCESS_FLAGS_REMOTEWRITE 0x0a
#define ZXDH_ACCESS_FLAGS_BIND_WINDOW 0x10
#define ZXDH_ACCESS_FLAGS_ZERO_BASED 0x20
#define ZXDH_ACCESS_FLAGS_ALL 0x3f

#define ZXDH_OP_TYPE_NOP 0x00
#define ZXDH_OP_TYPE_SEND 0x01
#define ZXDH_OP_TYPE_SEND_WITH_IMM 0x02
#define ZXDH_OP_TYPE_SEND_INV 0x03
#define ZXDH_OP_TYPE_WRITE 0x04
#define ZXDH_OP_TYPE_WRITE_WITH_IMM 0x05
#define ZXDH_OP_TYPE_READ 0x06
#define ZXDH_OP_TYPE_BIND_MW 0x07
#define ZXDH_OP_TYPE_FAST_REG_MR 0x08
#define ZXDH_OP_TYPE_LOCAL_INV 0x09
#define ZXDH_OP_TYPE_UD_SEND 0x0a
#define ZXDH_OP_TYPE_UD_SEND_WITH_IMM 0x0b
#define ZXDH_OP_TYPE_REC 0x3e
#define ZXDH_OP_TYPE_REC_IMM 0x3f

#define ZXDH_FLUSH_MAJOR_ERR 1
#define ZXDH_RETRY_ACK_MAJOR_ERR 0x8
#define ZXDH_RETRY_ACK_MINOR_ERR 0xf3
#define ZXDH_TX_WINDOW_QUERY_ITEM_MINOR_ERR 0xf5

#define ZXDH_MAX_SQ_FRAG 31
#define ZXDH_MAX_SQ_INLINE_DATELEN_WITH_IMM 210

#define INLINE_DATASIZE_7BYTES 7
#define INLINE_DATASIZE_24BYTES 24
#define INLINE_FRAG_DATASIZE_31BYTES 31

#define INLINE_DATA_OFFSET_7BYTES 7
#define WQE_OFFSET_7BYTES 7
#define WQE_OFFSET_8BYTES 8
#define WQE_OFFSET_24BYTES 24

#define ZXDH_SQE_SIZE 4
#define ZXDH_RQE_SIZE 2

#define ZXDH_SRQ_INVALID_LKEY 0x100
#define ZXDH_SRQ_DB_INIT_VALUE 0x8000

#define ZXDH_WQEALLOC_WQE_DESC_INDEX GENMASK(31, 20)

enum zxdh_device_caps_const {
	ZXDH_WQE_SIZE = 4,
	ZXDH_SRQE_SIZE = 2,
	ZXDH_CQP_WQE_SIZE = 8,
	ZXDH_CQE_SIZE = 8,
	ZXDH_EXTENDED_CQE_SIZE = 8,
	ZXDH_AEQE_SIZE = 2,
	ZXDH_CEQE_SIZE = 1,
	ZXDH_CQP_CTX_SIZE = 8,
	ZXDH_SHADOW_AREA_SIZE = 8,
	ZXDH_GATHER_STATS_BUF_SIZE = 1024,
	ZXDH_MIN_IW_QP_ID = 0,
	ZXDH_QUERY_FPM_BUF_SIZE = 176,
	ZXDH_COMMIT_FPM_BUF_SIZE = 176,
	ZXDH_MAX_IW_QP_ID = 262143,
	ZXDH_MIN_CEQID = 0,
	ZXDH_MAX_CEQID = 1023,
	ZXDH_CEQ_MAX_COUNT = ZXDH_MAX_CEQID + 1,
	ZXDH_MIN_CQID = 0,
	ZXDH_MAX_CQID = 524287,
	ZXDH_MIN_AEQ_ENTRIES = 1,
	ZXDH_MAX_AEQ_ENTRIES = 524287,
	ZXDH_MIN_CEQ_ENTRIES = 1,
	ZXDH_MAX_CEQ_ENTRIES = 262143,
	ZXDH_MIN_CQ_SIZE = 1,
	ZXDH_MAX_CQ_SIZE = 1048575,
	ZXDH_DB_ID_ZERO = 0,
	ZXDH_MAX_WQ_FRAGMENT_COUNT = 13,
	ZXDH_MAX_SGE_RD = 13,
	ZXDH_MAX_OUTBOUND_MSG_SIZE = 2147483647,
	ZXDH_MAX_INBOUND_MSG_SIZE = 2147483647,
	ZXDH_MAX_PUSH_PAGE_COUNT = 1024,
	ZXDH_MAX_PE_ENA_VF_COUNT = 32,
	ZXDH_MAX_VF_FPM_ID = 47,
	ZXDH_MAX_SQ_PAYLOAD_SIZE = 2147483648,
	ZXDH_MAX_INLINE_DATA_SIZE = 217,
	ZXDH_MAX_WQ_ENTRIES = 32768,
	ZXDH_Q2_BUF_SIZE = 256,
	ZXDH_QP_CTX_SIZE = 256,
	ZXDH_MAX_PDS = 262144,
};

enum zxdh_addressing_type {
	ZXDH_ADDR_TYPE_ZERO_BASED = 0,
	ZXDH_ADDR_TYPE_VA_BASED = 1,
};

enum zxdh_flush_opcode {
	FLUSH_INVALID = 0,
	FLUSH_GENERAL_ERR,
	FLUSH_PROT_ERR,
	FLUSH_REM_ACCESS_ERR,
	FLUSH_LOC_QP_OP_ERR,
	FLUSH_REM_OP_ERR,
	FLUSH_LOC_LEN_ERR,
	FLUSH_FATAL_ERR,
	FLUSH_RETRY_EXC_ERR,
	FLUSH_MW_BIND_ERR,
	FLUSH_REM_INV_REQ_ERR,
};

enum zxdh_cmpl_status {
	ZXDH_COMPL_STATUS_SUCCESS = 0,
	ZXDH_COMPL_STATUS_FLUSHED,
	ZXDH_COMPL_STATUS_INVALID_WQE,
	ZXDH_COMPL_STATUS_QP_CATASTROPHIC,
	ZXDH_COMPL_STATUS_REMOTE_TERMINATION,
	ZXDH_COMPL_STATUS_INVALID_STAG,
	ZXDH_COMPL_STATUS_BASE_BOUND_VIOLATION,
	ZXDH_COMPL_STATUS_ACCESS_VIOLATION,
	ZXDH_COMPL_STATUS_INVALID_PD_ID,
	ZXDH_COMPL_STATUS_WRAP_ERROR,
	ZXDH_COMPL_STATUS_STAG_INVALID_PDID,
	ZXDH_COMPL_STATUS_RDMA_READ_ZERO_ORD,
	ZXDH_COMPL_STATUS_QP_NOT_PRIVLEDGED,
	ZXDH_COMPL_STATUS_STAG_NOT_INVALID,
	ZXDH_COMPL_STATUS_INVALID_PHYS_BUF_SIZE,
	ZXDH_COMPL_STATUS_INVALID_PHYS_BUF_ENTRY,
	ZXDH_COMPL_STATUS_INVALID_FBO,
	ZXDH_COMPL_STATUS_INVALID_LEN,
	ZXDH_COMPL_STATUS_INVALID_ACCESS,
	ZXDH_COMPL_STATUS_PHYS_BUF_LIST_TOO_LONG,
	ZXDH_COMPL_STATUS_INVALID_VIRT_ADDRESS,
	ZXDH_COMPL_STATUS_INVALID_REGION,
	ZXDH_COMPL_STATUS_INVALID_WINDOW,
	ZXDH_COMPL_STATUS_INVALID_TOTAL_LEN,
	ZXDH_COMPL_STATUS_RETRY_ACK_ERR,
	ZXDH_COMPL_STATUS_TX_WINDOW_QUERY_ITEM_ERR,
	ZXDH_COMPL_STATUS_UNKNOWN,
};

enum zxdh_cmpl_notify {
	ZXDH_CQ_COMPL_EVENT = 0,
	ZXDH_CQ_COMPL_SOLICITED = 1,
};

enum zxdh_qp_caps {
	ZXDH_WRITE_WITH_IMM = 1,
	ZXDH_SEND_WITH_IMM = 2,
	ZXDH_ROCE = 4,
	ZXDH_PUSH_MODE = 8,
};

enum zxdh_page_size {
	ZXDH_PAGE_SIZE_4K = 0,
	ZXDH_PAGE_SIZE_2M = 9,
	ZXDH_PAGE_SIZE_1G = 18,
};

struct zxdh_qp;
struct zxdh_cq;
struct zxdh_qp_init_info;
struct zxdh_cq_init_info;

struct zxdh_sge {
	zxdh_tagged_offset tag_off;
	__u32 len;
	zxdh_stag stag;
};

struct zxdh_ring {
	__u32 head;
	__u32 tail;
	__u32 size;
};

struct zxdh_cqe {
	__le64 buf[ZXDH_CQE_SIZE];
};

struct zxdh_extended_cqe {
	__le64 buf[ZXDH_EXTENDED_CQE_SIZE];
};

struct zxdh_post_send {
	zxdh_sgl sg_list;
	__u32 num_sges;
	__u32 qkey;
	__u32 dest_qp;
	__u32 ah_id;
};

struct zxdh_inline_rdma_send {
	void *data;
	__u32 len;
	__u32 qkey;
	__u32 dest_qp;
	__u32 ah_id;
};

struct zxdh_post_rq_info {
	__u64 wr_id;
	zxdh_sgl sg_list;
	__u32 num_sges;
};

struct zxdh_rdma_write {
	zxdh_sgl lo_sg_list;
	__u32 num_lo_sges;
	struct zxdh_sge rem_addr;
};

struct zxdh_inline_rdma_write {
	void *data;
	__u32 len;
	struct zxdh_sge rem_addr;
};

struct zxdh_rdma_read {
	zxdh_sgl lo_sg_list;
	__u32 num_lo_sges;
	struct zxdh_sge rem_addr;
};

struct zxdh_bind_window {
	zxdh_stag mr_stag;
	__u64 bind_len;
	void *va;
	enum zxdh_addressing_type addressing_type;
	__u8 ena_reads : 1;
	__u8 ena_writes : 1;
	zxdh_stag mw_stag;
	__u8 mem_window_type_1 : 1;
	__u8 host_page_size;
	__u8 leaf_pbl_size;
	__u16 root_leaf_offset;
	__u64 mw_pa_pble_index;
};

struct zxdh_inv_local_stag {
	zxdh_stag target_stag;
};

struct zxdh_post_sq_info {
	__u64 wr_id;
	__u8 op_type;
	__u8 l4len;
	__u8 signaled : 1;
	__u8 read_fence : 1;
	__u8 local_fence : 1;
	__u8 inline_data : 1;
	__u8 imm_data_valid : 1;
	__u8 push_wqe : 1;
	__u8 report_rtt : 1;
	__u8 udp_hdr : 1;
	__u8 defer_flag : 1;
	__u8 solicited : 1;
	__u32 imm_data;
	__u32 stag_to_inv;
	union {
		struct zxdh_post_send send;
		struct zxdh_rdma_write rdma_write;
		struct zxdh_rdma_read rdma_read;
		struct zxdh_bind_window bind_window;
		struct zxdh_inv_local_stag inv_local_stag;
		struct zxdh_inline_rdma_write inline_rdma_write;
		struct zxdh_inline_rdma_send inline_rdma_send;
	} op;
};

struct zxdh_cq_poll_info {
	__u64 wr_id;
	zxdh_qp_handle qp_handle;
	__u32 bytes_xfered;
	__u32 tcp_seq_num_rtt;
	__u32 qp_id;
	__u32 ud_src_qpn;
	__u32 imm_data;
	zxdh_stag inv_stag; /* or L_R_Key */
	enum zxdh_cmpl_status comp_status;
	__u16 major_err;
	__u16 minor_err;
	__u8 op_type;
	__u8 stag_invalid_set : 1; /* or L_R_Key set */
	__u8 push_dropped : 1;
	__u8 error : 1;
	__u8 solicited_event : 1;
	__u8 ipv4 : 1;
	__u8 imm_valid : 1;
};

enum zxdh_status_code zxdh_inline_rdma_write(struct zxdh_qp *qp,
					     struct zxdh_post_sq_info *info,
					     bool post_sq);
enum zxdh_status_code zxdh_rc_inline_send(struct zxdh_qp *qp,
					  struct zxdh_post_sq_info *info,
					  bool post_sq);
enum zxdh_status_code zxdh_ud_inline_send(struct zxdh_qp *qp,
					  struct zxdh_post_sq_info *info,
					  bool post_sq);
enum zxdh_status_code
zxdh_mw_bind(struct zxdh_qp *qp, struct zxdh_post_sq_info *info, bool post_sq);
enum zxdh_status_code zxdh_post_nop(struct zxdh_qp *qp, __u64 wr_id,
				    bool signaled, bool post_sq);
enum zxdh_status_code zxdh_post_receive(struct zxdh_qp *qp,
					struct zxdh_post_rq_info *info);
void zxdh_qp_post_wr(struct zxdh_qp *qp);
void zxdh_qp_set_shadow_area(struct zxdh_qp *qp);
enum zxdh_status_code zxdh_rdma_read(struct zxdh_qp *qp,
				     struct zxdh_post_sq_info *info,
				     bool inv_stag, bool post_sq);
enum zxdh_status_code zxdh_rdma_write(struct zxdh_qp *qp,
				      struct zxdh_post_sq_info *info,
				      bool post_sq);
enum zxdh_status_code
zxdh_rc_send(struct zxdh_qp *qp, struct zxdh_post_sq_info *info, bool post_sq);
enum zxdh_status_code
zxdh_ud_send(struct zxdh_qp *qp, struct zxdh_post_sq_info *info, bool post_sq);
enum zxdh_status_code zxdh_stag_local_invalidate(struct zxdh_qp *qp,
						 struct zxdh_post_sq_info *info,
						 bool post_sq);

struct zxdh_wqe_ops {
	void (*iw_copy_inline_data)(__u8 *dest, __u8 *src, __u32 len,
				    __u8 polarity, bool imm_data_flag);
	__u16 (*iw_inline_data_size_to_quanta)(__u32 data_size,
					       bool imm_data_flag);
	void (*iw_set_fragment)(__le64 *wqe, __u32 offset, struct zxdh_sge *sge,
				__u8 valid);
	void (*iw_set_mw_bind_wqe)(__le64 *wqe,
				   struct zxdh_bind_window *op_info);
};

__le64 *get_current_cqe(struct zxdh_cq *cq);
enum zxdh_status_code zxdh_cq_poll_cmpl(struct zxdh_cq *cq,
					struct zxdh_cq_poll_info *info);
void zxdh_cq_request_notification(struct zxdh_cq *cq,
				  enum zxdh_cmpl_notify cq_notify);
void zxdh_cq_resize(struct zxdh_cq *cq, void *cq_base, int size);
void zxdh_cq_set_resized_cnt(struct zxdh_cq *qp, __u16 cnt);
enum zxdh_status_code zxdh_cq_init(struct zxdh_cq *cq,
				   struct zxdh_cq_init_info *info);
enum zxdh_status_code zxdh_qp_init(struct zxdh_qp *qp,
				   struct zxdh_qp_init_info *info);
struct zxdh_sq_wr_trk_info {
	__u64 wrid;
	__u32 wr_len;
	__u16 quanta;
	__u8 reserved[2];
};

struct zxdh_qp_sq_quanta {
	__le64 elem[ZXDH_SQE_SIZE];
};

struct zxdh_qp_rq_quanta {
	__le64 elem[ZXDH_RQE_SIZE];
};

struct zxdh_dev_attrs {
	__u64 feature_flags;
	__aligned_u64 sq_db_pa;
	__aligned_u64 cq_db_pa;
	__u32 max_hw_wq_frags;
	__u32 max_hw_read_sges;
	__u32 max_hw_inline;
	__u32 max_hw_rq_quanta;
	__u32 max_hw_srq_quanta;
	__u32 max_hw_wq_quanta;
	__u32 min_hw_cq_size;
	__u32 max_hw_cq_size;
	__u16 max_hw_sq_chunk;
	__u32 max_hw_srq_wr;
	__u8 hw_rev;
	__u8 db_addr_type;
};

struct zxdh_hw_attrs {
	struct zxdh_dev_attrs dev_attrs;
	__u64 max_hw_outbound_msg_size;
	__u64 max_hw_inbound_msg_size;
	__u64 max_mr_size;
	__u32 min_hw_qp_id;
	__u32 min_hw_aeq_size;
	__u32 max_hw_aeq_size;
	__u32 min_hw_ceq_size;
	__u32 max_hw_ceq_size;
	__u32 max_hw_device_pages;
	__u32 max_hw_vf_fpm_id;
	__u32 first_hw_vf_fpm_id;
	__u32 max_hw_ird;
	__u32 max_hw_ord;
	__u32 max_hw_wqes;
	__u32 max_hw_pds;
	__u32 max_hw_ena_vf_count;
	__u32 max_qp_wr;
	__u32 max_pe_ready_count;
	__u32 max_done_count;
	__u32 max_sleep_count;
	__u32 max_cqp_compl_wait_time_ms;
	__u16 max_stat_inst;
};

struct zxdh_qp {
	struct zxdh_qp_sq_quanta *sq_base;
	struct zxdh_qp_rq_quanta *rq_base;
	struct zxdh_dev_attrs *dev_attrs;
	__u32 *wqe_alloc_db;
	struct zxdh_sq_wr_trk_info *sq_wrtrk_array;
	__u64 *rq_wrid_array;
	__le64 *shadow_area;
	__le32 *push_db;
	__le64 *push_wqe;
	struct zxdh_ring sq_ring;
	struct zxdh_ring rq_ring;
	struct zxdh_ring initial_ring;
	__u32 qp_id;
	__u32 qp_caps;
	__u32 sq_size;
	__u32 rq_size;
	__u32 max_sq_frag_cnt;
	__u32 max_rq_frag_cnt;
	__u32 max_inline_data;
	struct zxdh_wqe_ops wqe_ops;
	__u16 conn_wqes;
	__u8 qp_type;
	__u8 swqe_polarity;
	__u8 swqe_polarity_deferred;
	__u8 rwqe_polarity;
	__u8 rq_wqe_size;
	__u8 rq_wqe_size_multiplier;
	__u8 deferred_flag : 1;
	__u8 push_mode : 1; /* whether the last post wqe was pushed */
	__u8 push_dropped : 1;
	__u8 sq_flush_complete : 1; /* Indicates flush was seen and SQ was empty after the flush */
	__u8 rq_flush_complete : 1; /* Indicates flush was seen and RQ was empty after the flush */
	__u8 destroy_pending : 1; /* Indicates the QP is being destroyed */
	void *back_qp;
	zxdh_sgl split_sg_list;
	pthread_spinlock_t *lock;
	__u16 rwqe_signature;
	__u8 dbg_rq_flushed;
	__u8 sq_flush_seen;
	__u8 rq_flush_seen;
	__u8 is_srq;
	__u16 mtu;
	__u32 next_psn;
	__u32 cqe_last_ack_qsn;
	__u32 qp_last_ack_qsn;
	__u8 cqe_retry_cnt;
	__u8 qp_reset_cnt;
};

struct zxdh_cq {
	struct zxdh_cqe *cq_base;
	__u32 *cqe_alloc_db;
	__u32 *cq_ack_db;
	__le64 *shadow_area;
	__u32 cq_id;
	__u32 cq_size;
	__u32 cqe_rd_cnt;
	struct zxdh_ring cq_ring;
	__u8 polarity;
	__u8 cqe_size;
};

struct zxdh_srq {
	struct zxdh_srq_wqe *srq_base;
	struct zxdh_dev_attrs *dev_attrs;
	__le16 *srq_list_base;
	__le64 *srq_db_base;
	__u32 srq_id;
	__u32 srq_size;
	__u32 log2_srq_size;
	__u32 srq_list_size;
	struct zxdh_ring srq_ring;
	struct zxdh_ring srq_list_ring;
	__u8 srq_list_polarity;
	__u64 *srq_wrid_array;
	__u8 srq_wqe_size;
	__u8 srq_wqe_size_multiplier;
	__u32 srq_caps;
	__u32 max_srq_frag_cnt;
	__u32 srq_type;
	pthread_spinlock_t *lock;
	__u8 srq_flush_complete : 1; /* Indicates flush was seen and SQ was empty after the flush */
	__u8 destroy_pending : 1; /* Indicates the QP is being destroyed */
	__u8 srq_flush_seen;
};

struct zxdh_qp_init_info {
	struct zxdh_qp_sq_quanta *sq;
	struct zxdh_qp_rq_quanta *rq;
	struct zxdh_dev_attrs *dev_attrs;
	__u32 *wqe_alloc_db;
	__le64 *shadow_area;
	struct zxdh_sq_wr_trk_info *sq_wrtrk_array;
	__u64 *rq_wrid_array;
	__u32 qp_id;
	__u32 qp_caps;
	__u32 sq_size;
	__u32 rq_size;
	__u32 max_sq_frag_cnt;
	__u32 max_rq_frag_cnt;
	__u32 max_inline_data;
	__u8 type;
	int abi_ver;
	bool legacy_mode;
};

struct zxdh_cq_init_info {
	__u32 *cqe_alloc_db;
	__u32 *cq_ack_db;
	struct zxdh_cqe *cq_base;
	__le64 *shadow_area;
	__u32 cq_size;
	__u32 cq_id;
	__u8 cqe_size;
};

struct zxdh_srq_init_info {
	struct zxdh_srq_wqe *srq_base;
	struct zxdh_dev_attrs *dev_attrs;
	__le16 *srq_list_base;
	__le64 *srq_db_base;
	__u64 *srq_wrid_array;
	__u32 srq_id;
	__u32 srq_caps;
	__u32 srq_size;
	__u32 log2_srq_size;
	__u32 srq_list_size;
	__u32 srq_db_size;
	__u32 max_srq_frag_cnt;
	__u32 srq_limit;
};

struct zxdh_wqe_srq_next_sge {
	__le16 next_wqe_index;
	__le16 signature;
	__u8 valid_sge_num;
	__u8 rsvd[11];
};

struct zxdh_srq_sge {
	__le64 addr;
	__le32 length;
	__le32 lkey;
};

struct zxdh_srq_wqe {
	__le64 elem[ZXDH_SRQE_SIZE];
};

__le64 *zxdh_qp_get_next_send_wqe(struct zxdh_qp *qp, __u32 *wqe_idx,
				  __u16 quanta, __u32 total_size,
				  struct zxdh_post_sq_info *info);
__le64 *zxdh_qp_get_next_recv_wqe(struct zxdh_qp *qp, __u32 *wqe_idx);
void zxdh_clean_cq(void *q, struct zxdh_cq *cq);
enum zxdh_status_code zxdh_nop(struct zxdh_qp *qp, __u64 wr_id, bool signaled,
			       bool post_sq);
enum zxdh_status_code zxdh_fragcnt_to_quanta_sq(__u32 frag_cnt, __u16 *quanta);
enum zxdh_status_code zxdh_fragcnt_to_wqesize_rq(__u32 frag_cnt,
						 __u16 *wqe_size);
void zxdh_get_sq_wqe_shift(__u32 sge, __u32 inline_data, __u8 *shift);
void zxdh_get_rq_wqe_shift(__u32 sge, __u8 *shift);
enum zxdh_status_code zxdh_get_sqdepth(struct zxdh_dev_attrs *dev_attrs,
				       __u32 sq_size, __u8 shift,
				       __u32 *wqdepth);
enum zxdh_status_code zxdh_get_rqdepth(struct zxdh_dev_attrs *dev_attrs,
				       __u32 rq_size, __u8 shift,
				       __u32 *wqdepth);
int zxdh_qp_round_up(__u32 wqdepth);
int zxdh_cq_round_up(__u32 wqdepth);
void zxdh_qp_push_wqe(struct zxdh_qp *qp, __le64 *wqe, __u16 quanta,
		      __u32 wqe_idx, bool post_sq);
void zxdh_clr_wqes(struct zxdh_qp *qp, __u32 qp_wqe_idx);

void zxdh_get_srq_wqe_shift(struct zxdh_dev_attrs *dev_attrs, __u32 sge,
			    __u8 *shift);
int zxdh_get_srqdepth(__u32 max_hw_srq_quanta, __u32 srq_size, __u8 shift,
		      __u32 *srqdepth);
__le64 *zxdh_get_srq_wqe(struct zxdh_srq *srq, int wqe_index);
__le16 *zxdh_get_srq_list_wqe(struct zxdh_srq *srq, __u16 *idx);

enum zxdh_status_code zxdh_srq_init(struct zxdh_srq *srq,
				    struct zxdh_srq_init_info *info);
void zxdh_free_srq_wqe(struct zxdh_srq *srq, int wqe_index);
#endif /* ZXDH_USER_H */
